"""
Simple concise program plugins (SCPP).

- This module focuses on UI/UX, aiming for readability and simplicity.
- This module is used along with other mykit-program related tools, like the ones from `cli_parser`, etc.

[20230919] Note for my future self: You will and should regroup the entire `/kit/` master module to make all of these more sense.
"""
