"""
This module is a build module for constructing mykit-related software/programs/libraries distribution.

Note, for distribution, this 'inventory' master module shouldn't be included since it has nothing to do with the end user.
"""
