# import TurboTest as tt

# from mykit.kit.fileops.nice_json import NiceJSON

# import os
# import tempfile


# def _prepare():
#     pass

# def _clear():
#     pass


# def read_existing_file():
#     _prepare()
#     _clear()


# def read_nonexisting_file():
#     pass


# def bar():
    
#     tt.both_are_equal(myfunc(1, 2), 3)



# """

# [21:12:33] PASS: mykit/kit/path: read existing file  (120 ms)
# [21:12:33] FAIL: mykit/kit/path: read nonexisting file  (12 ms)
# [21:12:33] FAIL: mykit/kit/foo/bar: Test core 123  (3.2 secs)
# [21:12:33] FAIL: mykit/kit/foo/bar: Test core 123  (3 mins)

# """